c       neutral dual absorber 
c photoelectric absorption and Compton + Rayleigh scattering 
c solved for transmission in 1d two-stream approximation (mu=+-1) 

	subroutine xsabscdt(ear,ne,param,Ifl,photar)
c
c     PARAMETERS
c               1       nh  - Hydrogen column of the absorper in units of 10^22
c               2       nhp - column density of partial covering absorber  
c               3       fr  - covering fraction 
c               4       ab0 - iron abundance relative to the 
c                             solar iron abundance
c               5       z   - redshift 
c	algorithm for a single absorber 
c  a(x) = a(x)*{ sh(2*k*tau_e)  / sh[k(tau+2*tau_e)]  }
c  where
c   k=sqrt(1-lambda)
c   lambda=sigmasc/(sigmasc+sigmaabs) - albedo for a single scattering
c   tau_e=ln[ (1+k)/(1-k) ] /(2k)
c   tau - total (absorption + scattering) optical depth 
c         sigmaabs   - photoelectric absorption cross-section
c         sigmasc    - Compton + Rayleigh scattering cross-section
c
        implicit    none
cjp 
c	integer*4   i,ne,n,nl
	integer   i,ne,n,nl
        integer     in,nn,Ifl     
        real*4      ear(0:ne),param(*),photar(ne)
	real        nh,nhp,z_red,ab0,x(5000),xnor,fr,fc 
	real        sigbfrd,sigmacsn,sigmatot
        real*8      tau,taup,lambda,transm 
        real        xh,xl,sph,spl,ah,al,enhp,enh 

c	set up constants
        nh=param(1)
        nhp=param(2) 
        z_red=param(5)
        ab0=log10(param(4))
        fr=param(3) 
        fc=1-fr 

c derredening of energy from the Earth frame
        n=ne+1
        nn=n
        do i=1,n
           in=i  
           x(in)=ear(i-1)/511.*(1+z_red)
        enddo

c  calculate the absorpted spectrum at all grid enegies

c  set the first low bin value
        in=1  
        xl=x(in)
        sigmatot=sigbfrd(in,x,nn,ab0,xnor)+sigmacsn(in,x,nn)
        lambda=sigmacsn(in,x,nn)/sigmatot
        taup=(nh+nhp)*.006652*sigmatot
        enhp=transm(taup,lambda)
        tau=nh*.006652*sigmatot
        enh=transm(tau,lambda)
        al=fc*enh+fr*enhp 
c       for averaging, assume the spectrum with Gamma=2
        spl=xl**(-2)

	do i=1,ne,1
c  limits for absorption significance
           if((xl.gt..019).and.(al.gt..998))then
              goto 10
           else
c  derredened high bin border
              in=i+1  
              xh=x(in)
c  power law spectrum at high bin border
              sph=xh**(-2)
c  absorption at high bin border
        sigmatot=sigbfrd(in,x,nn,ab0,xnor)+sigmacsn(in,x,nn) 
        taup=(nh+nhp)*.006652*sigmatot
        enhp=transm(taup,lambda)
        tau=nh*.006652*sigmatot
        enh=transm(tau,lambda)
        ah=fc*enh+fr*enhp 
c  multiplicative binned model 
              photar(i)=(sph*ah+spl*al)/(sph+spl)
c  set low bin value
              xl=xh
              al=ah
              spl=sph
           endif
	enddo
 10     nl=i
        do i=nl,ne,1
           photar(i)=1.
        enddo

	return
	end 

c ------------------------------------------------------------------- c
 

