c       neutral absorber + Compton + Rayleigh scattering 
c solved for transmission in 1d two-stream approximation (mu=+-1)
c no frequency redistribution 
c 
	subroutine xsabscst(ear,ne,param,Ifl,photar)
c
c     PARAMETERS
c               1       nh - Hydrogen column of the absorper in units of 10^22
c               2       z     - redshift
c               3       ab0   - iron abundance relative to the 
c                               solar iron abundance
c	algorithm: 
c  a(x) = a(x)*{ sh(2*k*tau_e)  / sh[k(tau+2*tau_e)]  }
c  where 
c   k=sqrt(1-lambda) 
c   lambda=sigmasc/(sigmasc+sigmaneb) - albedo for a single scattering 
c   tau_e=ln[ (1+k)/(1-k) ] /(2k)
c   tau - total (absorption + scattering) optical depth 
c         sigmaneb   - photoelectric absorption cross-section
c         sigmasc    - Compton + Rayleigh scattering cross-section
c
        implicit    none
cjp integer*4   i,ne,n,nl
	integer   i,ne,n,nl
        integer     in,nn,Ifl    
        real*4      ear(0:ne),param(*),photar(ne)
	real        nh,z_red,ab0,x(5000),xnor
        real        sigmaneb,sigmacsn,sigmatot
        real*8      lambda,tau,transm  
        real        xh,xl,sph,spl,ah,al

c	set up constants
        nh=param(1)
        z_red=param(2)
        ab0=log10(param(3))

c derredening of energy from the Earth frame
        n=ne+1
        nn=n
        do i=1,n
           in=i  
           x(in)=ear(i-1)/511.*(1+z_red)
        enddo

c  calculate the absorpted spectrum at all grid enegies

c  set the first low bin value
        in=1  
        xl=x(in) 
        sigmatot=sigmaneb(in,x,nn,ab0,xnor)+sigmacsn(in,x,nn)
        lambda=sigmacsn(in,x,nn)/sigmatot 
        tau=(nh*.006652*sigmatot) 
        al=transm(tau,lambda)
c       for averaging, assume the spectrum with Gamma=2
        spl=xl**(-2)

	do i=1,ne,1
c  limits for absorption significance
           if((xl.gt..019).and.(al.gt..998))then
              goto 10
           else
c  derredened high bin border
              in=i+1  
              xh=x(in)
c  power law spectrum at high bin border
              sph=xh**(-2)
c  absorption at high bin border
               sigmatot=sigmaneb(in,x,nn,ab0,xnor)+sigmacsn(in,x,nn)
        lambda=sigmacsn(in,x,nn)/sigmatot
        tau=(nh*.006652*sigmatot)
        ah=transm(tau,lambda)
c  multiplicative binned model 
              photar(i)=(sph*ah+spl*al)/(sph+spl)
c  set low bin value
              xl=xh
              al=ah
              spl=sph
           endif
	enddo
 10     nl=i
        do i=nl,ne,1
           photar(i)=1.
        enddo

	return
	end 

c ------------------------------------------------------------------- c

      function sigmacsn(in,x,n)
c    Compton scattering cross section for neutral matter with cosmic abundances 
c    per hydrogen atom in units of the Thomson cross section
c    only neutral  hydrogen and helium are accounted for 
c     x - energy vector of the length n; 
c     sigmacsn is calculated for  element in from x
      implicit real (a-h,o-z)
      implicit integer (i-n)
      real x(5000),xold(5000),opcs(5000)
      logical fl1
      save opcs,xold,nold,fl1
      data xold /5000*0./
      data fl1/.true./
      data nold /0/ 

c     if the energy vector changed from the
c     previous call, recompute the opacity (in abscorn) and reset xold
      if((x(in).ne.xold(in)).or.(n.ne.nold)) then
          fl1=.true.
          nold=n 
          do 11 i=1,n
            xold(i)=x(i)
 11      continue
      end if
c     If the energy vector changed,  recompute the opacity
      if(fl1) then 
        call abscron(x,n,opcs) 
        fl1= .false.
      endif

      sigmacsn=opcs(in)

      return
      end

c**************************************************
c Compton scattering cross-section for neutral H and He 
      subroutine abscron(xx,ii,sc) 
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER(NCRO=24) 
      real xx(5000),xkev(5000),sc(5000)  
      real*8 CRRAT(5000),CRCOH(5000),CRINC(5000),CRCPT(5000) 
      DIMENSION HYDINC(NCRO),HELINC(NCRO),HYDCOH(NCRO),HELCOH(NCRO),
     1 XFEK(NCRO)
      DATA CTHOM/0.6652D0/,EPS/1D-14/,HELHYD/0.1D0/   
      IONIZ=0 
c if IONIZ=0 then 
c cross-section for coherent and incoherent scattering (Rayleigh and 
c Compton) from Hubbell et al. (1975) J.Phys.Chem.Ref.Data, 4, No.3, 
c p.471-538 Table II.  
c HYDCOH - coherent   cross-section for hydrogen 
c HYDINC - incoherent cross-section for hydrogen 
c HELCOH - coherent   cross-section for helium    
c HELINC - incoherent cross-section for helium  

c if IONIZ=1 then 
c HELCOH=0    and  HYDCOH=0  and  CRCOH=0  
c HELINC=2*CS and  HYDINC=CS and  CRINC=CS*(1+2*HELHYD) 
c where CS=COMPTON SCATTERING CROSS-SECTION (KLEIN-NISHINA) 
c CS=CRCPT in this program 

      DATA HYDCOH/0.6650D0,0.6635D0,0.6617D0,0.6569D0,
     1 0.6503D0,0.6421D0,0.6323D0,0.6087D0,0.5806D0,
     2 0.4984D0,0.4142D0,0.2766D0,0.1881D0,0.1341D0,
     3 0.9987D-1,0.6126D-1,0.4121D-1,0.1943D-1,0.1119D-1, 
     4 0.5062D-2,0.2866D-2,0.1840D-2,0.1280D-2,0.7211D-3/

      DATA HYDINC/0.9552D-3,0.2144D-2,0.3802D-2,0.8494D-2,
     1 0.1496D-1,0.2310D-1,0.3279D-1,0.5629D-1,0.8424D-1,
     2 0.1650D0,0.2478D0,0.3822D0,0.4675D0,0.5182D0,
     3 0.5503D0,0.5840D0,0.5993D0,0.6095D0,0.6068D0, 
     4 0.5924D0,0.5759D0,0.5597D0,0.5444D0,0.5166D0/
     
      DATA HELCOH/2.660D0,2.658D0,2.655D0,2.648D0,
     1 2.637D0,2.624D0,2.608D0,2.567D0,2.517D0,  
     2 2.356D0,2.162D0,1.743D0,1.369D0,1.072D0,
     3 0.8492D0,0.5592D0,0.3921D0,0.1962D0,0.1166D0, 
     4 0.5432D-1,0.3113D-1,0.2010D-1,0.1403D-1,0.7933D-2/ 

      DATA HELINC/0.7194D-3,0.1618D-2,0.2876D-2,0.6438D-2,
     1 0.1144D-1,0.1768D-1,0.2523D-1,0.4413D-1,0.6759D-1,
     2 0.1419D0,0.2317D0,0.4230D0,0.5948D0,0.7328D0,
     3 0.8385D0,0.9785D0,0.1059D1,0.1145D1,0.1168D1, 
     4 0.1163D1,0.1139D1,0.1111D1,0.1083D1,0.1030D1/

      DATA XFEK/0.1D0,0.15D0,0.2D0,0.3D0,0.4D0,0.5D0,0.6D0,0.8D0, 
     1 0.1D1,0.15D1,0.2D1,0.3D1,0.4D1,0.5D1,0.6D1,0.8D1, 
     2 0.1D2,0.15D2,0.2D2,0.3D2,0.4D2,0.5D2,0.6D2,0.8D2/ 
     
      DLN=DLOG(1D1) 
      DO 20 IX=1,II     
      X=XX(IX) 
      XEV=XX(IX)*511.D0 
      XKEV(IX)=XEV  
c cross-section for cold electron gas 
      CALL ASN(X,0,SSP,SSP1,SSP2,SP1,SP2,SP3,SP4,SP5,SP6,SP7,EPS) 
      CRCPT(IX)=SSP 
      IF(IONIZ.eq.0) THEN 
c neutral hydrogen and helium 
c calculation of the helium/hydrogen  cross-sections ratio CRRAT 
c coherent cross-section CRCOH, incoherent cross-section CRINC  
      IF(XEV .le. XFEK(1)) THEN 
      FRAT=(HELCOH(1)+HELINC(1))/(HYDCOH(1)+HYDINC(1)) 
      FCOHYD=CTHOM
      FINHYD=0D0  
      FCOHEL=4D0*CTHOM 
      FINHEL=0D0  
      ELSE IF(XEV .ge. XFEK(NCRO)) THEN 
      FRAT=2D0 
      FCOHYD=0D0  
      FINHYD=CRCPT(IX)*CTHOM 
      FCOHEL=0D0  
      FINHEL=2D0*CRCPT(IX)*CTHOM  
      ELSE 
      CALL AHUNT(XFEK,NCRO,XEV,ICRO)
      XL=XFEK(ICRO) 
      T=(XEV-XL)/(XFEK(ICRO+1)-XL) 

      YCOHYD1=HYDCOH(ICRO)
      YCOHYD2=HYDCOH(ICRO+1)
      FCOHYD=YCOHYD1+T*(YCOHYD2-YCOHYD1) 

      YINHYD1=HYDINC(ICRO)
      YINHYD2=HYDINC(ICRO+1)
      FINHYD=YINHYD1+T*(YINHYD2-YINHYD1) 

      YCOHEL1=HELCOH(ICRO)
      YCOHEL2=HELCOH(ICRO+1)
      FCOHEL=YCOHEL1+T*(YCOHEL2-YCOHEL1) 

      YINHEL1=HELINC(ICRO)
      YINHEL2=HELINC(ICRO+1)
      FINHEL=YINHEL1+T*(YINHEL2-YINHEL1) 

      YRAT1=(YCOHEL1+YINHEL1)/(YCOHYD1+YINHYD1) 
      YRAT2=(YCOHEL2+YINHEL2)/(YCOHYD2+YINHYD2) 
      FRAT=YRAT1+T*(YRAT2-YRAT1) 
      ENDIF 

      ELSE 
c ionized hydrogen and helium 
      FRAT=2D0 
      FCOHYD=0D0  
      FINHYD=CRCPT(IX)*CTHOM 
      FCOHEL=0D0  
      FINHEL=2D0*CRCPT(IX)*CTHOM  
      ENDIF 

c ratio of helium to hydrogen cross-sections 
      CRRAT(IX)=FRAT 
c total coherent cross-section 
      CRCOH(IX)=(FCOHYD+HELHYD*FCOHEL)/CTHOM 
c total incoherent cross-section 
      CRINC(IX)=(FINHYD+HELHYD*FINHEL)/CTHOM 
c total scattering cross-section 
      SC(IX)=CRCPT(IX)*(1D0+HELHYD*FRAT) 
c renormalization of CRCOH and CRINC to SC 
      CRCOH(IX)=CRCOH(IX)*SC(IX)/(CRCOH(IX)+CRINC(IX)) 
      CRINC(IX)=CRINC(IX)*SC(IX)/(CRCOH(IX)+CRINC(IX)) 
   20 CONTINUE 
      RETURN 
      END 


c 1d two-stream approximation for trasmission
      function transm(tau,lambda)
      implicit none
      real*8 lambda,tau,transm
      real*8 accur,term,arg,taue,sum3
      real*8 ekmtau
      real*8 k,km,kp,ktau,kmt,kpt,ratk,ratk2,eps
      integer*4  n
      data eps/1d-4/,accur/1d-14/

      k=dsqrt(1d0-lambda)
      kp=1d0+k
      km=1d0-k
      kmt=km*tau
      kpt=kp*tau
      ratk=km/kp
      ratk2=ratk*ratk
      ktau=k*tau

      if(ktau.lt.7d1) then
      ekmtau=dexp(-ktau)
      else
      ekmtau=0d0
      endif

      if(k.lt.eps.and.ktau.lt.eps) then

      taue=0d0
      arg=k*k
      term=1d0
      n=1
        do while(dabs(term).gt.accur)
        taue=taue+term/dfloat(n)
        term=term*arg
        n=n+2
        enddo


      sum3=0d0
      arg=k*(tau+2d0*taue)
      arg=arg*arg
      term=1d0
      n=1
        do while(dabs(term).gt.accur)
        sum3=sum3+term
        n=n+2
        term=term*arg/dfloat(n*(n-1))
        enddo

       transm=2d0/(lambda*(tau+2d0*taue)*sum3)

************
      else

      transm=(1d0-ratk2)*ekmtau/(1d0-ratk2*ekmtau*ekmtau)

      endif 

      RETURN
      END

c**********************************************************************
      SUBROUTINE AHUNT(XX,N,X,JLO)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION XX(N)
      LOGICAL ASCND
      ASCND=XX(N).GT.XX(1)
      IF(JLO.LE.0.OR.JLO.GT.N)THEN
        JLO=0
        JHI=N+1
        GO TO 3
      ENDIF
      INC=1
      IF(X.GE.XX(JLO).EQV.ASCND)THEN
1       JHI=JLO+INC
        IF(JHI.GT.N)THEN
          JHI=N+1
        ELSE IF(X.GE.XX(JHI).EQV.ASCND)THEN
          JLO=JHI
         INC=INC+INC
          GO TO 1
        ENDIF
      ELSE
        JHI=JLO
2       JLO=JHI-INC
        IF(JLO.LT.1)THEN
          JLO=0
        ELSE IF(X.LT.XX(JLO).EQV.ASCND)THEN
          JHI=JLO
          INC=INC+INC
          GO TO 2
        ENDIF
      ENDIF
3     IF(JHI-JLO.EQ.1)RETURN
      JM=(JHI+JLO)/2
      IF(X.GT.XX(JM).EQV.ASCND)THEN
        JLO=JM
      ELSE
        JHI=JM
      ENDIF
      GO TO 3
      END

C  S- functions. if K=0, then only  SS -- mean cross-section;
C if  K=1, then  SS1 &  S1 -- mean frequency;
C if K=2, all functions -- and the square of frequency
C x -- small
      SUBROUTINE ASN(X,K,SS,SS1,SS2,S1,S2,S3,S4,S5,S6,S7,E)
      IMPLICIT REAL*8(A-H,O-Z)
      DATA XT /4D-1/
      DATA C13/0.333333333333333D0/
      IF (X.LT.XT) GO TO 110
      X0=1D0/(1D0+2D0*X)
      X2=X*X
      XR=1D0/X
      DLX0=-DLOG(X0)
      X02=X0*X0
      X2R=XR*XR
      SS=0.375D0*(4D0-(2D0*XR+2D0-X)*DLX0+2D0*X2*(1D0+X)*X02)*X2R
      IF(K.EQ.0) RETURN
      SS1=0.125D0*X2R*XR*(3D0*DLX0+4D0*X2-4.5D0*X-X*X0*(1.5D0+X*X02))
      S1=(SS-SS1)*XR
      S2=(SS1-S1)*XR
      IF(K.EQ.1) RETURN
            SS2=(1D0+X*(4D0+X*(7D0+4.5D0*X)))*X02*X02
      S3=(SS1-SS2)*XR
            S4=(S1-S3)*XR
      GO TO 160
  110 A=1D0
      B=-2D0*X
      DN=0D0
      SS=0D0
      IF (K.EQ.0) GO TO 120
      SS1=0D0
      S1=0D0
      S2=0D0
      IF (K.EQ.1) GO TO 120
      SS2=0D0
      S3=0D0
      S4=0D0
  120 DN1=DN+1D0
      DN2=DN+2D0
      DN3=DN+3D0
      D1=1D0/DN1
      D2=1D0/DN2
      D3=1D0/DN3
      DN4=DN+4D0
      DN23=DN2*DN3
      DN234=0.25D0*DN23*DN4
      SS=SS+A*(DN2+2D0*D1+8D0*D2-16D0*D3)
      IF (K.EQ.0) GO TO 140
            D4=1D0/DN4
      D5=1D0/(DN4+1D0)
            DN13=DN1*DN3
                  DN14=DN1*DN4
      SS1=SS1+A*(DN23-6D0+24D0*D3)
      S1=S1+A*(DN13-6D0-6D0*D2-24D0*D3+72D0*D4)
      S2=S2+A*(DN14-6D0-12D0*D3-72D0*D4+144D0*D5)
      IF (K.EQ.1) GO TO 140
      DN134=0.25D0*DN13*DN4
      DN124=0.25D0*DN2*DN14
      SS2=SS2+A*(DN234+2D0-DN) 
      S3=S3+A*(DN134+7D0-DN-24D0*D4)
      S4=S4+A*(DN124+12D0-DN+6D0*D3+24D0*D4-96D0*D5)
  140 IF (DABS(4D0*A)*DN234.LT.E) GO TO 150
      DN=DN+1D0
      A=A*B
      GO TO 120
  150 SS=SS*0.375D0
      IF (K.EQ.0) RETURN
      SS1=SS1*0.125D0
      S1=S1*0.25D0
      S2=S2*0.25D0
      IF (K.EQ.1) RETURN
            SS2=SS2*0.125D0
      S3=S3*0.25D0
      S4=S4*0.5D0
  160 S5=3D0*S4-4D0*S3
      S7=S3-0.5D0*S4
      S6=2D0*SS2-6D0*S7
      RETURN
      END


c ------------------------------------------------------------------- c
c abundances due to  Anders & Ebihara 
c 
      real function sigmaneb(in,x,n,ab0,xnor)
c     bound-free cross section for neutral matter with cosmic abundances
c     per hydrogen atom in units of the Thomson cross section
c     ab0 - iron log10 abundance relative to hydrogen
c     x - energy vector of the length n; sigmaneb is calculated for
c     element in from x
      implicit none  
      integer in,n
      real x(5000),ab0,xnor
      integer i,nold
      real abh,xold(5000),opac(5000),ab(17)
      real totlneb
      logical fl1,fl2
      save opac,ab,xold,nold,fl1,fl2
      data xold /5000*0./
      data nold /0/ 
      data(ab(i),i=1,17)/12.,11.00,8.65,7.96,8.87,8.14,6.32,7.60,6.49,
     &                     7.57,7.28,5.28,6.58,6.35,5.69,999999.,6.26/
 
      abh=ab0+7.52
c     if the energy vector changed from the
c     previous call, recompute the opacity (in totlxs0) and reset xold
      if((x(in).ne.xold(in)).or.(n.ne.nold)) then
          fl2=.true.
          nold=n 
          do 11 i=1,n
            xold(i)=x(i)
 11      continue
      end if
c     If either the iron abundance or the energy vector changed,
c     recompute the opacity
      if(ab(16).ne.abh) then
         fl1=.true.
         ab(16)=abh
      endif
      if(fl1.or.fl2) then
         do 10 i=1,n
            opac(i)=totlneb(ab,i,x,n,fl1,fl2,xnor)
 10      continue
      endif

      sigmaneb=opac(in)
 
      return
      end
 
 
C----------------------------------------------------------------------
C
      FUNCTION TOTLNEB( AB ,in,x,n,fl1,fl2,xnor)
c     *** this version with fully ionized H and He!  AAZ, 18.05.94
c     Optimized version: calculetes cross section for abundances 
c     contained in the vector AB at energy in 511.keV units given as
c     an element numbered in of the vector x; n is the length of x.
c     Flag fl1=.true. must indicate changes of AB, and fl2=.true.
c     changes of the energy vector x. For energy >10.keV the law
c     xnor*e^-3 is used, xnor is an output parameter.
c     Note: result in sigma_Thomson units.
C
C     Reference:
C      Monika Balucinska-Church and Dan McCammon
C      "Photoelectric Absorption Cross Sections with Variable Abundances"
C      Ap.J. 400, 699 (1992)
C
C
C     Description:
C     Calculates the effective absorption cross section in units of
c     sigma_Thomson per hydrogen atom at energy E in eV for the 
c     abundances of the elements specified in vector AB
C
C     Method:
C     Calls seventeen functions that calculate the mass absorption coeffs
C     in cm**2/g for the following elements: H, He, C, N, O, Ne, Na, Mg,
C     Al, Si, S, Cl, Ar, Ca, Cr, Fe, Ni.
C
C     Deficiencies:
C     Works only in the range of energy from 30 eV to 10,000 eV.
C
C     Bugs:
C     None known -- please report any problems to authors
C
C     Authors:
C     Dan McCammon               (47413::MCCAMMON)
C     Monika Balucinska-Church   (19775::MBC)
C
C     History:
C     8.8.91  - original
C     8.1.92  - modified to remove VAX fortran77 extensions
C     12.3.93 - function HELIUM has been replaced by HELIUM_V2 with
C               Fano line profile
C     21.9.93 - Names of functions simplified: TOTLXS_V2 to TOTLX2,
C               HELIUM_V2 to HEL2.
C     24.1.94 - Names of functions simplified: TOTLX2 to TOTLXS,
C               HEL2 to HELIUM.
C
C     Parameters:
C     E - energy in eV
C     AB - vector of length 17 of Log10 abundances relative to
C          hydrogen = AB(1).  N.B. The order of the elements in AB
C          is as follows: H,He,C,N,O,Ne,Na,Mg,Al,Si,S,Cl,Ar,Ca,Cr,Fe,Ni.
C     TOTLX2 - effective cross section in cm**2/H atom for assumed
C             abundances of cosmic elements.  Note that this will be per
C             hydrogen atom for the relative abundances given in AB.
C
C     Type Definitions:
      IMPLICIT NONE
      integer i
      real xnor,xnors 
C
C     Local variables:
      INTEGER K
C          ( index through elements)
      REAL A(17,5000),x(5000),toth(5000),ap(17)
      integer in,n
c          x  - energy points
c          in - index for x
c          n  - number of energy points
C          ( mass absorption cross sections in cm**2/g
C           for the cosmic elements -- in the same order as for AB)
C     Import:
      REAL E
C          ( energy in eV)
      REAL AB(17)
C          ( log10 abundances relative to hydrogen)
 
C     Export:
      REAL TOTLNEB 
C          ( effective cross section in cm**2/H atom)
C     External functions: ( mass absorption coefficients for the elements)
      REAL HYDRO0
      REAL HELIUM0
      REAL CARBON0
      REAL NITRO0
      REAL OXYGEN0
      REAL NEON0
      REAL SODIUM0
      REAL MAGNES0
      REAL ALUM0
      REAL SILICN0
      REAL SULFUR0
      REAL CHLORN0
      REAL ARGON0
      REAL CALC0
      REAL CHROM0
      REAL IRON0
      REAL NICKEL0
 
 
C     Local constants:
      REAL AW(17)
C          ( atomic weights of the elements)
      REAL AV
C          ( Avogadro's number) <- multiplied by sigma_Thomson
      INTEGER NUMB
C          ( number of elements)
      integer inh
c          ( limiting bin for asymptotic absorption )
 
      logical fl0,fl1,fl2
      save a,aw,av,numb,ap,fl0,xnors
 
      DATA AW /1.00797, 4.0026, 12.01115, 14.0067, 15.9994, 20.183,
     &         22.9898, 24.312, 26.9815, 28.086, 32.064, 35.453,
     &         39.94, 40.08, 51.996, 55.847, 58.71/
 
c      DATA AV /6.022045E23/
      DATA AV /.400614/

      DATA NUMB /17/

      data fl0/.true./

C     Start:
 
c     normalizing factor for the asymptotic relation
      if(fl1)then
C        mass absorption coefficients for the elements at 10.keV
         if(fl0)then
            E=10000.
            ap( 1)=HYDRO0(E)  
            ap( 2)=HELIUM0(E)
c            ap( 1)=0
c            ap( 2)=0
            ap( 3)=CARBON0(E)
            ap( 4)=NITRO0(E)
            ap( 5)=OXYGEN0(E)
            ap( 6)=NEON0(E)
            ap( 7)=SODIUM0(E)
            ap( 8)=MAGNES0(E)
            ap( 9)=ALUM0(E)
            ap(10)=SILICN0(E)
            ap(11)=SULFUR0(E)
            ap(12)=CHLORN0(E)
            ap(13)=ARGON0(E)
            ap(14)=CALC0(E)
            ap(15)=CHROM0(E)
            ap(16)=IRON0(E)
            ap(17)=NICKEL0(E)
            fl0=.false.
         endif
         xnors=0.
         do 30 k=1,numb
            xnors=xnors+AW(K)*ap(K)*(10.**(AB(K)-AB(1)))
 30      continue
         xnors=xnors/av*(0.01947)**3
         fl1=.false.
      endif
      xnor=xnors
 
      if(fl2) then
         do 20 i=1,n
c           above 10 keV, use e**-3 law
            if(x(i).lt.0.01947) then
               E=x(i)*511000
               A( 1,i)=HYDRO0(E)  
               A( 2,i)=HELIUM0(E)
c               A( 1,i)=0
c               A( 2,i)=0
               A( 3,i)=CARBON0(E)
               A( 4,i)=NITRO0(E)
               A( 5,i)=OXYGEN0(E)
               A( 6,i)=NEON0(E)
               A( 7,i)=SODIUM0(E)
               A( 8,i)=MAGNES0(E)
               A( 9,i)=ALUM0(E)
               A(10,i)=SILICN0(E)
               A(11,i)=SULFUR0(E)
               A(12,i)=CHLORN0(E)
               A(13,i)=ARGON0(E)
               A(14,i)=CALC0(E)
               A(15,i)=CHROM0(E)
               A(16,i)=IRON0(E)
               A(17,i)=NICKEL0(E)
               inh=i
            else
               toth(i)=x(i)**(-3)
            endif
 20      continue
         inh=inh+1
         fl2=.false.
      endif

      if(x(in).lt..01947)then
         totlneb=0.
C        loop through elements
         DO 10 K=1,NUMB
            totlneb=totlneb+AW(K)*A(K,in)*(10.**(AB(K)-AB(1)))
10       CONTINUE
         totlneb=totlneb/av
      else
         totlneb=xnor*toth(in)
      endif
 
      RETURN
      END
 
 
C-------------------------------------------------------------------------

