      subroutine xstransm(ear,ne,param,Ifl,photar)  
c     version: 24 April 1998 
c     changed to XSPEC10 version
c     version: 01 April 1996 
c     Driver for angle-independent transmission through the slab 
c     from an exponentially-cutoff power law with Compton reflection
c     (Magdziarz & Zdziarski 1995)  and neutral medium.  
c     The intrinsic spectrum is incident along the normal. 
c
c     The output spectrum is just the transmitted flux.
c
c     If E_c=0, there is no cutoff in the power law.
c
c     Version with cosmic iron abundance and new opacities of Balucinska
c     & McCammon (1992, and 1994, private communication). As expected in AGNs,
c     H and He are assumed to be fully ionized.
c     This version allows for changes of the vector 'ear' between subsequent
c       calls.
c
c
c     number of model parameters: 6
c Absorbing medium: 
c     1: Column density of hydrogen (in 10^22 cm^-2) 
c Intrinsic spectrum: 
c     2: E_c, the cutoff energy in keV (if E_c=0 there is no cutoff;
c        one needs to change the lower limit for that)
c     3: Gamma, power law photon index, N_E prop. to E^{-Gamma} 
c     4: scale, scaling factor for reflection; if <0, no direct component
c        (scale=1 for isotropic source above disk) 
c     5: cosine of inclination angle 
c     6: redshift, z
c     algorithm:
c          a[E*(1+z)]=using Green's function for transmission 
c     Normalization is the photon flux at 1 keV (photons keV^-1 cm^-2 s^-1)
c     of the power law only    and in the earth frame.
c
c
      implicit none
      external powrefa,powerlaw  
      integer *4 ne,i
      integer j,jmax,Ifl 
      REAL*4 ear(0:ne), param(*),photar(ne)
      real xz(5000)
      logical firstcall
      save firstcall
      data firstcall/.true./
 
      if (firstcall) then
        call xwrite('TEST PROGRAM for',5)
        call xwrite('Compton transmission through neutral medium',5)
        call xwrite('(except for ionized H and He)',5)
        call xwrite('for incident power law with exp. cutoff.',5)
        call xwrite('plus Compton reflection',5)
        call xwrite('foldE=0 gives no cutoff',5)
        call xwrite('rel_refl<0 gives as an intrinsic spectrum',5)
        call xwrite('the reflected spectrum alone',5)
        call xwrite('Questions: Juri Poutanen',5)
        call xwrite('juri@astro.su.se',5)
        firstcall = .false.
      end if
 
c
      do 10 i=0,ne
         j=i+1
c        ear and xz are in the Earth frame
         xz(j)=ear(i)/511.
 10   continue
      jmax=ne+1
c
c     xz is the energy array (units m_e c^2)
c     sptran is the transmitted spectrum array (E F_E)
c     dimension = jmax
      call comptran(param,xz,powrefa,jmax,photar) 
      return
      end
C--------------------------------------------------------------------------
      subroutine comptran(param,xz,funintr,jmax,photar) 
      implicit real (a-h,o-z)
      implicit integer (i-n) 
      parameter(nx=50,ntau=21,nlamb=16,ntaura=21,nste=5) 
      parameter(nnmax=nste+1) 
      common/trcom/trcomp(ntau,nx,nx,nste),tauco(ntau),xcomp(nx,nste),
     1 scco(nx,nste),phco(nx,nste),dste(nste),dste2(nste)
      common/trray/trrayl(ntaura,nlamb),tau(ntaura),dla(nlamb)  
      real sptran(5000),photaux(5000)
      real grray(5000),xzcomp(nx,nste)  
      real photar(*),xz(*),xx(5000),param(*),par(7)   
      real cpintr(nx,nste),cppow(nx,nste),cpref(nx,nste)
      real acomp(nx,nste),cptran(nx,nste)   
      real cptran1(nx,nste),cptran2(nx,nste),f2(nx)   
      real xold(5000),spintr(5000),sppow(5000),spref(5000)
      real spixbb(1),sprxbb(1),sppxbb(1)
      real dlamb(5000),sc(5000),phab(5000),sigmatot(5000),parold(6)  
      integer nna(nnmax)
      real xbb(nnmax),xbbz(nnmax)    
      external funintr 
      logical firstcall,fl1,fl2,fl3  
      save xold,nold,fl1,fl2,tauold,grray,parold   
      save firstcall,fl3
      save acomp,nbound,sc,phab,dlamb,cpintr,spintr   
      save grxbb,sigxbb,dlxbb,sptxbb  
      save nna,nbound1,xzcomp,sigmatot,spixbb,xx     
      data firstcall/.true./
      data xold /5000*0./,tauold/99999./,parold/6*99999./   
      data nold /0/,fl1/.false./,fl2/.true./,fl3/.true./  
c this is for dste=2**i, i=-1,..5, dste2=dste/8, nx=50, x=dste/(nx-1+dste2) 
c nste= 7
c      data xbb/0.0100883,0.0203563,0.04061,0.08081,0.16,0.313725,
c     + 0.603774,8./
c nste= 5
      data xbb/0.0100883,0.0203563,0.04061,0.08081,0.16,8./ 
c nste= 4 and without line
c      data xbb/0.0203563,0.04061,0.08081,0.16,8./ 
c this is for dste=2**i, i=0,..5, dste2=dste/8, nx=50, x=dste/(nx-1+dste2) 
c nste= 6
c      data xbb/0.0100883,0.0201512,0.040202,0.08,0.158416,0.31068,8./ 
      data xbound/0.0100883/,xmax/8./,xline/0.017/
c      data xbound/0.0201512/,xmax/8./,xline/0.017/
      data eps/1e-10/,ab0/0./,dln/2.3025851/    
      data cmin/1e-30/,dlmin/-30./,cmax/70./  

      z_red=param(6)

      par(1)=param(3) 
      par(2)=param(2) 
      par(3)=param(4) 
      par(4)=param(6) 
      par(5)=1. 
      par(6)=1. 
      par(7)=param(5) 
c optical depth tauh=n_h*sigma_T*H=N_h*sigma_T 
      tauh=param(1)*0.006652

c  if the energy vector changed from the
c  previous call, recompute all cross-sections and matrices and reset xold
      do 11 i=1,jmax 
      if((xz(i).ne.xold(i)).or.(jmax.ne.nold)) then
          fl1=.true.
          nold=jmax 
          xold(i)=xz(i)
      end if
 11   continue

c deredening of energy from the Earth frame
c     xz, xx  are the energy array (units m_e c^2)
      if(fl1.or.firstcall.or.param(6).ne.parold(6)) then 
      do 30 j=1,jmax
         xx(j)=xz(j)*(1+z_red)
 30   continue
c deviding all frequencies to two bands: 
c Rayleigh part with lg(x)<lg(0.0203)
c compton part other 
      do 8 i=1,nste  
      call clocat(xx,jmax,xbb(i),nna(i)) 
    8 continue
      call clocat(xx,jmax,xbb(nnmax),nna(nnmax)) 
      nbound=nna(1) 
      nbound1=MIN0(jmax,nbound+1) 
      do i=1,nnmax 
      xbbz(i)=xbb(i)/(1.+z_red)
      enddo 
      endif 

      if (firstcall) then 
      call readtran 
      do 16 iste=1,nste  
c photoelectric absorption at Compton part
c right now this corresponds to neutral matter H and He also
      call sigmapha(xcomp,phco,ab0,xnor,iste,nx)
   16 continue 
      DO 4 iste=1,nste  
      do 4 j=1,nx 
      x=xcomp(j,iste)
      acomp(j,iste)=x*x/dste(iste)  
c Compton cross-section
      call csn(x,0,SSP,SSP1,SSP2,SP1,SP2,SP3,SP4,SP5,SP6,SP7,EPS)
      scco(j,iste)=ssp
   4  continue 
      endif 

      if(firstcall.or.param(6).ne.parold(6)) then 
      do 5 iste=1,nste  
      do 5 j=1,nx 
      xzcomp(j,iste)=xcomp(j,iste)/(1.+z_red)  
5     continue 
      endif 

      if (firstcall.or.fl1) then 
c rayleigh part 
      if(nbound.gt.0) then 
c photoelectric absorption at Rayleigh part
c right now this corresponds to neutral matter
c but for ionized H and He 
      call sigmapha(xx,phab,ab0,xnor,1,nbound1)
      do 10 j=1,nbound1
c Compton cross-section 
      call csn(xx(j),0,SSP,SSP1,SSP2,SP1,SP2,SP3,SP4,SP5,SP6,SP7,EPS) 
cjp270596 : *1.2 for ionized H and He
      sc(j)=ssp*1.2
c total attenuation 
      sigmatot(j)=sc(j)+phab(j) 
c albedo for single scattering  
      dlamb(j)=sc(j)/sigmatot(j)
10    continue 
      endif 
c reading transmission matrices for all optical depths (Rayleigh) 
c      if(nbound.gt.0)  call raylmatr
      end if

      if (firstcall) then 
c compute Rayleigh transmision for x=xbb(1) 
      call sigmapha(xbb,phab,ab0,xnor,1,1)
      call csn(xbb(1),0,SSP,SSP1,SSP2,SP1,SP2,SP3,SP4,SP5,SP6,SP7,EPS) 
      scxbb=ssp*1.2
      sigxbb=scxbb+phab(1) 
      dlxbb=scxbb/sigxbb 
      end if

        if(firstcall.or.parold(6).ne.param(6)) then 
        do 67 iste=1,nste 
        do 67 i=1,nx 
        xzcomp(i,iste)=xcomp(i,iste)/(1.+z_red)
67      continue 
        endif 

      
c if tauh has not changed do not recompute matrices 
      if(tauold.ne.tauh) then
          fl2=.true.
          tauold=tauh 
      end if 

      if(fl2.or.fl1) then 
c computation of the tr. matrices for a given tau by interpolation 
c Rayleigh part
      if(nbound.gt.0) then
      do 43 j=1,nbound1  
c      grray(j)=rayint(tauh*sc(j),dlamb(j))  
cjp 260596
      grray(j)=tranone(tauh*sigmatot(j),dlamb(j)) 
43    continue 

      endif 
c Compton part 
c   
      end if

c transmition at xbb(1) 
      if(fl2) then 
      grxbb=tranone(tauh*sigxbb,dlxbb) 
      end if

c comparison with old spectral parameters 
      do 21 ipa=2,6 
      if(parold(ipa).ne.param(ipa)) fl3=.true. 
      parold(ipa)=param(ipa)
 21   continue  

c calculation of the transmitted spectrum at xbb(1) 
      if(fl3.or.param(6).ne.parold(6)) 
     & call funintr(par,xbbz,1,spixbb,sprxbb,sppxbb) 
      sptxbb=spixbb(1)*grxbb    

c calculation of the transmitted spectrum 
c Rayleigh part: do it if min(xx) < xbound 
      if(nbound.gt.0) then 
cjp
c      if(fl3) call funintr(par,xz,nbound1,spintr,sppow,spref)
      call funintr(par,xz,nbound1,spintr,sppow,spref)
      do 60 j=1,nbound1 
      sptran(j)=spintr(j)*grray(j) 
      photaux(j)=sptran(j)/(511.*xz(j))**2
60    continue 

c calculating photon flux in the bins (Rayleigh) 
      do 63 j=1,nbound-1  
      photar(j)=.5*(photaux(j+1)+photaux(j))*(xz(j+1)-xz(j))*511. 
63    continue 
cjp   
c      j=nbound
c      phnorm=.5*(photaux(j+1)+photaux(j))*(xz(j+1)-xz(j))*511.

      endif 

      call clocat(xx,jmax,xline,nbline) 
      nbomax=nna(nnmax) 
c Compton part: do it if max(xx) > xbound 
c if 100 
      if(nbound.lt.jmax) then 

      iste=1
c while 110 
      do while (jmax.gt.nna(iste).and.iste.le.nste) 

c input spectrum E F_E 
      if(fl3.or.fl1.or.param(6).ne.parold(6)) 
     1 call funintr(par,xzcomp(1,iste),nx,cpintr(1,iste),
     2 cppow(1,iste),cpref(1,iste))

      call clocat(tauco,ntau,tauh,ktau)  

c transmitted spectrum  F_E/E 
      do 45 i=1,nx 
      cross=scco(i,iste)+phco(i,iste)
      tautot=tauh*cross
      sum1=0. 
      sum2=0. 
       do 55 i1=i,nx 
       x1=xcomp(i1,iste) 
       x12=x1*x1 
cjp280596: devide by 2 since we forgot to do so for tran*_norm.50_3 
c       weight=acomp(i1,iste)*cpintr(i1,iste)/x12
cjp100998 devide by 2 only old versions of matr.dat 
       weight=acomp(i1,iste)*cpintr(i1,iste)/x12/2. 
       if(ktau.ne.0) sum1=sum1+trcomp(ktau,i,i1,iste)*weight
       if(ktau.ne.ntau) sum2=sum2+trcomp(ktau+1,i,i1,iste)*weight
55     continue
c transmitted spectrum F_E/E for tau_1 < tauh 
      cptran1(i,iste)=sum1
c transmitted spectrum F_E/E for tau_2 > tauh 
      cptran2(i,iste)=sum2
c interpolation 
      if(ktau.eq.0) then
      tt=(tauco(1)-tauh)*cross 
      ett=0.
      if(tt.lt.cmax) ett=exp(tt)
      cptran(i,iste)=sum2*ett
      endif 

      if(ktau.eq.ntau) then
      tt=(tauco(ntau)-tauh)*cross 
      ett=0.
      if(tt.lt.cmax) ett=exp(tt)
      cptran(i,iste)=sum1*ett
      endif 
    
      if(ktau.ne.0.and.ktau.ne.ntau) then  
      tauk=alog(tauco(ktau))
      tauk1=alog(tauco(ktau+1))
      tt=(alog(tauh)-tauk)/(tauk1-tauk) 
      cptran(i,iste)=(sum1+tt*(sum2-sum1))
      endif 

45    continue

c end of while loop 110
      iste=iste+1 
      enddo 
cjp
c transmitted spectrum at original grid 
      iste=1 
c here loop for integration from xx(n) to xx(n+1); n=nbound+1,...,jmax-1 
cjp
      goto 333 
      do j=nbound,nbline+1 
c      do j=nbound+1,nbline+1 
      xmin=xx(j)
      xmax=xx(j+1)
      call chunt(xcomp(1,iste),nx,xmin,jlo) 
      nmin=jlo
      call chunt(xcomp(1,iste),nx,xmax,jlo) 
      nmax=jlo
      call binsum(cptran(1,iste),sum,xmin,xmax,nmin,nmax,iste)
      photar(j)=sum/511. 
      enddo 
 333  continue 

c      imin=nbline+2 
      imin=nbound   
      imax=nbomax-1
      call cspline(xcomp(1,iste),cptran(1,iste),nx,1e30,1e30,f2)
      do j=imin,imax
      xmin=xx(j)
      if(xmin.ge.xbb(iste+1)) then
      iste=iste+1
      call cspline(xcomp(1,iste),cptran(1,iste),nx,1e30,1e30,f2)
      endif 
      xmax=xx(j+1)
      call binint(xcomp(1,iste),cptran(1,iste),f2,nx,xmin,xmax,sum)
      photar(j)=sum/511. 
      enddo 

      do j=nbomax,jmax-1 
      photar(j)=0. 
      enddo

c compute transmitted at xbb(1) using CS part
      call csplint(xcomp(1,1),cptran(1,1),f2,nx,xbb(1),FUNCX)

c renormalization of Compton part to be consistent with Rayleigh part 
      phnorm=sptxbb/funcx/xbb(1)**2.  
cjp100998 testing other way around 
c normalizing 
c      do 65 j=imin,imax 
c      photar(j)=photar(j)*phnorm 
c   65 continue

c renormalization of Rayleigh part to be consistent with Compton part 
cjp100998 this line I cannot understand 
c      phnorm=photar(nbound)/phnorm
      phnorm=1./phnorm
      do 65 j=1,nbound-1
      photar(j)=photar(j)*phnorm 
65    continue

c end if 100 
      endif 

      firstcall = .false.
      fl2=.false. 
      fl1=.false. 
      fl3=.false. 

      return
      end
 
C--------------------------------------------------------------------------
      subroutine readtran
      implicit real (a-h,o-z)
      parameter(nx=50,ntau=21,nlamb=16,ntaura=21,nste=5) 
      common/trcom/trcomp(ntau,nx,nx,nste),tauco(ntau),xcomp(nx,nste),
     1 scco(nx,nste),phco(nx,nste),dste(nste),dste2(nste) 

      integer lenact,ilun,ios
      character*72 absfil
cjp for xspec10 
      include '../spectral/inc/sitedef.inc'

c initial values for tr.matr.       
      do 15 iste=1,nste
      do 15 i=1,nx 
      do 30 itau=1,ntau
      do 30 i1=1,nx 
      trcomp(itau,i,i1,iste)=0. 
   30 continue 
      xcomp(i,iste)=0.
   15 continue 

c      open(40,file='transm.dat',form='formatted',status='old')
c loop in DSTE: DSTE=2**(ids-1), DSTE2=DSTE/8. 
      do 40 ids=1,nste
      call getlun(ilun)

c      if(nx.eq.50) then
      if(ids-2.eq.-1) then
      absfil= XSPECDR(:lenact(XSPECDR))//'matr-1.dat'
      call openwr(ilun, absfil, 'old', ' ', ' ', 0, 0, ios) 
c      OPEN (40,FILE='TRANSM_DATA/tran-1_norm.50_3')
      elseif(ids-2.eq.0) then
c      if(ids-1.eq.0) then
      absfil= XSPECDR(:lenact(XSPECDR))//'matr0.dat'
      call openwr(ilun, absfil, 'old', ' ', ' ', 0, 0, ios) 
c      OPEN (40,FILE='TRANSM_DATA/tran0_norm.50_3')
      elseif(ids-2.eq.1) then
      absfil= XSPECDR(:lenact(XSPECDR))//'matr1.dat'
      call openwr(ilun, absfil, 'old', ' ', ' ', 0, 0, ios) 
c      OPEN (40,FILE='TRANSM_DATA/tran1_norm.50_3')
      elseif(ids-2.eq.2) then
      absfil= XSPECDR(:lenact(XSPECDR))//'matr2.dat' 
      call openwr(ilun, absfil, 'old', ' ', ' ', 0, 0, ios) 
c      OPEN (40,FILE='TRANSM_DATA/tran2_norm.50_3')
      elseif(ids-2.eq.3) then
      absfil= XSPECDR(:lenact(XSPECDR))//'matr3.dat'
      call openwr(ilun, absfil, 'old', ' ', ' ', 0, 0, ios) 
c      OPEN (40,FILE='TRANSM_DATA/tran3_norm.50_3')
      elseif(ids-2.eq.4) then
      absfil= XSPECDR(:lenact(XSPECDR))//'matr4.dat'
      call openwr(ilun, absfil, 'old', ' ', ' ', 0, 0, ios) 
c      OPEN (40,FILE='TRANSM_DATA/tran4_norm.50_3')
      elseif(ids-2.eq.5) then
      absfil= XSPECDR(:lenact(XSPECDR))//'matr5.dat'
      call openwr(ilun, absfil, 'old', ' ', ' ', 0, 0, ios) 
c      OPEN (40,FILE='TRANSM_DATA/tran5_norm.50_3')
      endif
c      endif

c loop in tau 
      DO 20 ITAU=1,NTAU 

      READ(ilun,3331) tauco(itau),II,NC,DSTE(ids),DSTE2(ids) 
c loop in frequencies, read the transm. matrix at lin-grid  
      DO 523 I=1,II
      READ(ilun,3332) xcomp(i,ids) 
      READ(ilun,3333) (trcomp(itau,i,i1,ids),i1=i,ii)
  523 CONTINUE


   20 continue
c end of loop in tau
      close(ilun)
      call frelun(ilun) 
c      close(UNIT=40)
   40 continue 
c end of loop in DSTE 


 3331 FORMAT(1x,E14.6,2I5,2E14.6)
 3332 FORMAT(1x,E14.6)
 3333 FORMAT(1x,5E14.6)
      
      return
      end
C--------------------------------------------------------------------------

c**********************************************************************
      SUBROUTINE CHUNT(XX,N,X,JLO)
      IMPLICIT REAL (A-H,O-Z)
      DIMENSION XX(N)
      LOGICAL ASCND
      ASCND=XX(N).GT.XX(1)
      IF(JLO.LE.0.OR.JLO.GT.N)THEN
        JLO=0
        JHI=N+1
        GO TO 3
      ENDIF
      INC=1
      IF(X.GE.XX(JLO).EQV.ASCND)THEN
1       JHI=JLO+INC
        IF(JHI.GT.N)THEN
          JHI=N+1
        ELSE IF(X.GE.XX(JHI).EQV.ASCND)THEN
          JLO=JHI
          INC=INC+INC
          GO TO 1
        ENDIF
      ELSE
        JHI=JLO
2       JLO=JHI-INC
        IF(JLO.LT.1)THEN 
          JLO=0
        ELSE IF(X.LT.XX(JLO).EQV.ASCND)THEN
          JHI=JLO
          INC=INC+INC
          GO TO 2
        ENDIF
      ENDIF
3     IF(JHI-JLO.EQ.1)RETURN
      JM=(JHI+JLO)/2
      IF(X.GT.XX(JM).EQV.ASCND)THEN
        JLO=JM
      ELSE
        JHI=JM
      ENDIF
      GO TO 3
      END

c**************************************************
      SUBROUTINE CLOCAT(XX,N,X,J)
      IMPLICIT REAL (A-H,O-Z)
      DIMENSION XX(N)
      JL=0
      JU=N+1
10    IF(JU-JL.GT.1)THEN
        JM=(JU+JL)/2
        IF((XX(N).GT.XX(1)).EQV.(X.GT.XX(JM)))THEN
          JL=JM
        ELSE
          JU=JM
        ENDIF
      GO TO 10
      ENDIF
      J=JL
      RETURN
      END
c**************************************************
C  S- functions. if K=0, then only  SS -- mean cross-section;
C if  K=1, then  SS1 &  S1 -- mean frequency;
C if K=2, all functions -- and the square of frequency
C x -- small
      SUBROUTINE CSN(X,K,SS,SS1,SS2,S1,S2,S3,S4,S5,S6,S7,E)
      IMPLICIT REAL  (A-H,O-Z)
      DATA XT /4E-1/
      DATA C13/0.333333333333333E0/
      IF (X.LT.XT) GO TO 110
      X0=1E0/(1E0+2E0*X)
      X2=X*X
      XR=1E0/X
      DLX0=-ALOG(X0)
      X02=X0*X0
      X2R=XR*XR
      SS=0.375E0*(4E0-(2E0*XR+2E0-X)*DLX0+2E0*X2*(1E0+X)*X02)*X2R
      IF(K.EQ.0) RETURN
      SS1=0.125E0*X2R*XR*(3E0*DLX0+4E0*X2-4.5E0*X-X*X0*(1.5E0+X*X02))
      S1=(SS-SS1)*XR
      S2=(SS1-S1)*XR
      IF(K.EQ.1) RETURN
            SS2=(1E0+X*(4E0+X*(7E0+4.5E0*X)))*X02*X02
      S3=(SS1-SS2)*XR
            S4=(S1-S3)*XR
      GO TO 160
  110 A=1E0
      B=-2E0*X
      DN=0E0
      SS=0E0
      IF (K.EQ.0) GO TO 120
      SS1=0E0
      S1=0E0
      S2=0E0
      IF (K.EQ.1) GO TO 120
      SS2=0E0
      S3=0E0
      S4=0E0
  120 DN1=DN+1E0
      DN2=DN+2E0
      DN3=DN+3E0
      D1=1E0/DN1
      D2=1E0/DN2
      D3=1E0/DN3
      DN4=DN+4E0
      DN23=DN2*DN3
      DN234=0.25E0*DN23*DN4
      SS=SS+A*(DN2+2E0*D1+8E0*D2-16E0*D3)
      IF (K.EQ.0) GO TO 140
            D4=1E0/DN4
      D5=1E0/(DN4+1E0)
            DN13=DN1*DN3
                  DN14=DN1*DN4
      SS1=SS1+A*(DN23-6E0+24E0*D3)
      S1=S1+A*(DN13-6E0-6E0*D2-24E0*D3+72E0*D4)
      S2=S2+A*(DN14-6E0-12E0*D3-72E0*D4+144E0*D5)
      IF (K.EQ.1) GO TO 140
      DN134=0.25E0*DN13*DN4
      DN124=0.25E0*DN2*DN14
      SS2=SS2+A*(DN234+2E0-DN)
      S3=S3+A*(DN134+7E0-DN-24E0*D4)
      S4=S4+A*(DN124+12E0-DN+6E0*D3+24E0*D4-96E0*D5)
  140 IF (ABS(4E0*A)*DN234.LT.E) GO TO 150
      DN=DN+1E0
      A=A*B
      GO TO 120
  150 SS=SS*0.375E0
      IF (K.EQ.0) RETURN
      SS1=SS1*0.125E0
      S1=S1*0.25E0
      S2=S2*0.25E0
      IF (K.EQ.1) RETURN
            SS2=SS2*0.125E0
      S3=S3*0.25E0
      S4=S4*0.5E0
  160 S5=3E0*S4-4E0*S3
      S7=S3-0.5E0*S4
      S6=2E0*SS2-6E0*S7
      RETURN
      END

c integrating inside the bin (NOT averaging) 
      subroutine binsum(cptran,sum,xmin,xmax,nmin,nmax,nds)
      implicit real (a-h,o-z)
      parameter(nx=50,ntau=21,nlamb=16,ntaura=21,nste=5) 
      common/trcom/trcomp(ntau,nx,nx,nste),tauco(ntau),xcomp(nx,nste),
     1 scco(nx,nste),phco(nx,nste),dste(nste),dste2(nste)
      real cptran(nx)  

        nmin1=nmin+1
        nmax1=nmax  
        dni=nx+dste2(nds)-dste(nds)/xmin
        dna=nx+dste2(nds)-dste(nds)/xmax
        if(nmin1-dni .gt. 0.5) nmin1=nmin1-1
        if(-nmax1+dna .gt. 0.5) nmax1=nmax1+1

      sum=0.
      do 210 i1=nmin1,nmax1 
      ai=1.
      x1=xcomp(i1,nds)
      if(i1.eq.nmin1) ai=nmin1+0.5-dni
      if(i1.eq.nmax1) ai=-nmax1+0.5+dna
      if(nmin1.eq.nmax1) ai=dna-dni
      ai=ai*x1*x1
      sum=sum+cptran(i1)*ai
  210 continue
      sum=sum/dste(nds)

      return
      end


      subroutine binint(xcomp,cptran,f2,nx,A,B,S)
      parameter (eps=1.e-4, jmax=20)
      implicit real (a-h,o-z)
      real cptran(nx),xcomp(nx),f2(nx)   

      OLDS=-1.E30
      DO 11 J=1,JMAX
      call CTRAPZD(xcomp,cptran,f2,nx,A,B,S,J)      
        IF (ABS(S-OLDS).LT.EPS*ABS(OLDS)) RETURN
        OLDS=S
11    CONTINUE
c      PAUSE 'Too many steps in binint'
      end


      SUBROUTINE CTRAPZD(xcomp,cptran,f2,nx,A,B,S,N)
      implicit real (a-h,o-z) 
      real cptran(nx),xcomp(nx),f2(nx) 

      IF (N.EQ.1) THEN
      call csplint(xcomp,cptran,f2,nx,A,FUNCA) 
      call csplint(xcomp,cptran,f2,nx,B,FUNCB) 
        S=0.5*(B-A)*(FUNCA+FUNCB)
        IT=1
      ELSE
        TNM=IT
        DEL=(B-A)/TNM
        X=A+0.5*DEL
        SUM=0.
        DO 11 J=1,IT
      call csplint(xcomp,cptran,f2,nx,X,FUNCX) 
          SUM=SUM+FUNCX
          X=X+DEL
11      CONTINUE
        S=0.5*(S+(B-A)*SUM/TNM)
        IT=2*IT
      ENDIF
      RETURN
      END


      SUBROUTINE CSPLINE(X,Y,N,YP1,YPN,Y2)
      implicit real (a-h,o-z) 
      PARAMETER (NMAX=100)
      DIMENSION X(N),Y(N),Y2(N),U(NMAX)
      IF (YP1.GT..99E30) THEN
        Y2(1)=0.
        U(1)=0.
      ELSE
        Y2(1)=-0.5
        U(1)=(3./(X(2)-X(1)))*((Y(2)-Y(1))/(X(2)-X(1))-YP1)
      ENDIF
      DO 11 I=2,N-1
        SIG=(X(I)-X(I-1))/(X(I+1)-X(I-1))
        P=SIG*Y2(I-1)+2.
        Y2(I)=(SIG-1.)/P
        U(I)=(6.*((Y(I+1)-Y(I))/(X(I+1)-X(I))-(Y(I)-Y(I-1))
     *      /(X(I)-X(I-1)))/(X(I+1)-X(I-1))-SIG*U(I-1))/P
11    CONTINUE
      IF (YPN.GT..99E30) THEN
        QN=0.
        UN=0.
      ELSE
        QN=0.5
        UN=(3./(X(N)-X(N-1)))*(YPN-(Y(N)-Y(N-1))/(X(N)-X(N-1)))
      ENDIF
      Y2(N)=(UN-QN*U(N-1))/(QN*Y2(N-1)+1.)
      DO 12 K=N-1,1,-1
        Y2(K)=Y2(K)*Y2(K+1)+U(K)
12    CONTINUE
      RETURN
      END

      SUBROUTINE CSPLINT(XA,YA,Y2A,N,X,Y)
      implicit real (a-h,o-z) 
      DIMENSION XA(N),YA(N),Y2A(N)
      KLO=1
      KHI=N
1     IF (KHI-KLO.GT.1) THEN
        K=(KHI+KLO)/2
        IF(XA(K).GT.X)THEN
          KHI=K
        ELSE
          KLO=K
        ENDIF
      GOTO 1
      ENDIF
      H=XA(KHI)-XA(KLO)
      IF (H.EQ.0.) PAUSE 'Bad XA input.'
      A=(XA(KHI)-X)/H
      B=(X-XA(KLO))/H
      Y=A*YA(KLO)+B*YA(KHI)+
     *      ((A**3-A)*Y2A(KLO)+(B**3-B)*Y2A(KHI))*(H**2)/6.
      RETURN
      END

      subroutine sigmapha(xx,phel,ab0,xnor,iste,n)
c      function sigmapha(x,ab0,xnor,n,in)
c     bound-free cross section for neutral matter with cosmic abundances
c     per hydrogen atom in units of the Thomson cross section
c     ab0 - iron log10 abundance relative to hydrogen
c     x - energy vector of the length n; sigmabf is calculated for
c     element in from x
      implicit real (a-h,o-z)
      implicit integer (i-n)
      real x(5000),xold(5000),phel(1),ab(17),xx(1),opac(5000)  
      logical fl1,fl2
      save opac,ab,xold,nold,fl1,fl2,isteold 
      data xold /5000*0./
      data nold /0/,isteold/0/, fl2/.false./
      data(ab(i),i=1,17)/12.,11.,8.65,7.96,8.87,8.14,6.32,7.60,6.49,
     &                   7.57,7.28,5.28,6.58,6.35,5.69,999999.,6.26/

      do i=1,n 
      x(i)=xx(i+(iste-1)*n)
      enddo 

      abh=ab0+7.52
c     if the energy vector changed from the
c     previous call, recompute the opacity (in totlxs) and reset xold
      if(n.ne.nold.or.iste.ne.isteold) then 
      fl2=.true.
      nold=n 
      isteold=iste 
      end if

      do in=1,n 
      if(x(in).ne.xold(in)) then
          fl2=.true.
          xold(in)=x(in)
      end if
      enddo 
c     If either the iron abundance or the energy vector changed,
c     recompute the opacity
      if(ab(16).ne.abh) then
         fl1=.true.
         ab(16)=abh
      endif
      if(fl1.or.fl2) then
         do 10 i=1,n
            opac(i)=totlxsa(ab,i,x,n,fl1,fl2,xnor)
 10      continue
      endif
      do i=1,n 
      phel(i+(iste-1)*n)=opac(i)
      enddo 

      return
      end



c 1d two-stream approximation for trasmission
      function tranone(tau,lambda)
      implicit none
      real lambda,tau,tranone
      real accur,term,arg,taue,sum3
      real ekmtau,cmin 
      real k,km,kp,ktau,kmt,kpt,ratk,ratk2,eps
      integer*4  n
      data eps/1e-4/,accur/1e-10/,cmin/1e-30/ 

      k=sqrt(1e0-lambda)
      kp=1e0+k
      km=1e0-k
      kmt=km*tau
      kpt=kp*tau
      ratk=km/kp
      ratk2=ratk*ratk
      ktau=k*tau

      if(ktau.lt.7e1) then
      ekmtau=exp(-ktau)
      else
      ekmtau=cmin 
      endif

      if(k.lt.eps.and.ktau.lt.eps) then

      taue=0e0
      arg=k*k
      term=1e0
      n=1
        do while(abs(term).gt.accur)
        taue=taue+term/float(n)
        term=term*arg
        n=n+2
        enddo


      sum3=0e0
      arg=k*(tau+2e0*taue)
      arg=arg*arg
      term=1e0
      n=1
        do while(abs(term).gt.accur)
        sum3=sum3+term
        n=n+2
        term=term*arg/float(n*(n-1))
        enddo

       tranone=2e0/(lambda*(tau+2e0*taue)*sum3)

************
      else

      tranone=(1e0-ratk2)*ekmtau/(1e0-ratk2*ekmtau*ekmtau)

      endif

      RETURN
      END

C--------------------------------------------------------------------------
C--------------------------------------------------------------------------

